@@ -7,8 +7,6 @@
 
 #include "orcus/css_selector.hpp"
 
-#include <cstring>
-
 namespace orcus {
 
 css_simple_selector_t::css_simple_selector_t() :
@@ -90,29 +88,31 @@ bool css_selector_t::operator== (const css_selector_t& r) const
 css_property_value_t::css_property_value_t() :
     type(css::property_value_t::none), str(nullptr) {}
 
-css_property_value_t::css_property_value_t(const css_property_value_t& r) :
-    type(r.type)
+namespace
 {
-    switch (type)
+
+void assign_value(css_property_value_t& v, const css_property_value_t& r)
+{
+    switch (r.type)
     {
         case css::property_value_t::rgb:
         case css::property_value_t::rgba:
-            red = r.red;
-            green = r.green;
-            blue = r.blue;
-            alpha = r.alpha;
+            v.red = r.red;
+            v.green = r.green;
+            v.blue = r.blue;
+            v.alpha = r.alpha;
         break;
         case css::property_value_t::hsl:
         case css::property_value_t::hsla:
-            hue = r.hue;
-            saturation = r.saturation;
-            lightness = r.lightness;
-            alpha = r.alpha;
+            v.hue = r.hue;
+            v.saturation = r.saturation;
+            v.lightness = r.lightness;
+            v.alpha = r.alpha;
         break;
         case css::property_value_t::string:
         case css::property_value_t::url:
-            str = r.str;
-            length = r.length;
+            v.str = r.str;
+            v.length = r.length;
         break;
         case css::property_value_t::none:
         default:
@@ -120,22 +120,35 @@ css_property_value_t::css_property_value_t(const css_property_value_t& r) :
     }
 }
 
+}
+
+css_property_value_t::css_property_value_t(const css_property_value_t& r) :
+    type(r.type)
+{
+    assign_value(*this, r);
+}
+
 css_property_value_t::css_property_value_t(const pstring& str) :
     type(css::property_value_t::string), str(str.get()), length(str.size()) {}
 
 css_property_value_t& css_property_value_t::operator= (const css_property_value_t& r)
 {
     if (&r != this)
-        std::memcpy(this, &r, sizeof(css_property_value_t));
+    {
+        type = r.type;
+        assign_value(*this, r);
+    }
     return *this;
 }
 
 void css_property_value_t::swap(css_property_value_t& r)
 {
-    unsigned char buf[sizeof(css_property_value_t)];
-    std::memcpy(buf, this, sizeof(css_property_value_t));
-    std::memcpy(this, &r, sizeof(css_property_value_t));
-    std::memcpy(&r, buf, sizeof(css_property_value_t));
+    if (&r != this)
+    {
+        css_property_value_t tmp(*this);
+        *this = r;
+        r = tmp;
+    }
 }
 
 std::ostream& operator<< (std::ostream& os, const css_simple_selector_t& v)
