@@ -333,23 +333,35 @@ void ods_content_xml_context::end_child_context(xmlns_id_t ns, xml_token_t name,
     }
     else if (ns == NS_odf_office && name == XML_automatic_styles)
     {
-        cout << "styles picked up:" << endl;
+        if (get_config().debug)
+            cout << "styles picked up:" << endl;
+
         odf_styles_map_type::const_iterator it = m_styles.begin(), it_end = m_styles.end();
         for (; it != it_end; ++it)
         {
-            cout << "  style: " << it->first << " [ ";
+            if (get_config().debug)
+                cout << "  style: " << it->first << " [ ";
+
             switch (it->second->family)
             {
                 case style_family_table_column:
-                    cout << "column width: " << it->second->column_data->width.print();
-                break;
+                {
+                    if (get_config().debug)
+                        cout << "column width: " << it->second->column_data->width.to_string();
+                    break;
+                }
                 case style_family_table_row:
-                    cout << "row height: " << it->second->row_data->height.print();
-                break;
+                {
+                    if (get_config().debug)
+                        cout << "row height: " << it->second->row_data->height.to_string();
+                    break;
+                }
                 case style_family_table_cell:
                 {
                     const odf_style::cell& cell = *it->second->cell_data;
-                    cout << "xf ID: " << cell.xf;
+                    if (get_config().debug)
+                        cout << "xf ID: " << cell.xf;
+
                     spreadsheet::iface::import_styles* styles = mp_factory->get_styles();
                     if (styles)
                     {
@@ -357,18 +369,23 @@ void ods_content_xml_context::end_child_context(xmlns_id_t ns, xml_token_t name,
                         //  currently we have no way to set a real style to a cell anyway
                         m_cell_format_map.insert(name2id_type::value_type(it->first, cell.xf));
                     }
+                    break;
                 }
-                break;
                 case style_family_text:
                 {
-                    const odf_style::text& data = *it->second->text_data;
-                    cout << "font ID: " << data.font;
+                    if (get_config().debug)
+                    {
+                        const odf_style::text& data = *it->second->text_data;
+                        cout << "font ID: " << data.font;
+                    }
+                    break;
                 }
-                break;
                 default:
                     ;
             }
-            cout << " ]" << endl;
+
+            if (get_config().debug)
+                cout << " ]" << endl;
         }
     }
 }
@@ -486,19 +503,25 @@ void ods_content_xml_context::start_table(const xml_attrs_t& attrs)
 {
     table_attr_parser parser = for_each(attrs.begin(), attrs.end(), table_attr_parser());
     const pstring& name = parser.get_name();
-    m_tables.push_back(mp_factory->append_sheet(name.get(), name.size()));
-    cout << "start table " << name << endl;
+    m_tables.push_back(mp_factory->append_sheet(m_tables.size(), name.get(), name.size()));
+
+    if (get_config().debug)
+        cout << "start table " << name << endl;
 
     m_row = m_col = 0;
 }
 
 void ods_content_xml_context::end_table()
 {
-    cout << "end table" << endl;
+    if (get_config().debug)
+        cout << "end table" << endl;
 }
 
 void ods_content_xml_context::start_column(const xml_attrs_t& attrs)
 {
+    if (!m_tables.back())
+        return;
+
     spreadsheet::iface::import_sheet_properties* sheet_props =
         m_tables.back()->get_sheet_properties();
 
@@ -530,6 +553,9 @@ void ods_content_xml_context::start_row(const xml_attrs_t& attrs)
     func = for_each(attrs.begin(), attrs.end(), func);
     m_row_attr.number_rows_repeated = func.get_number_rows_repeated();
 
+    if (!m_tables.back())
+        return;
+
     // Pass row properties to the interface.
     spreadsheet::iface::import_sheet_properties* sheet_props =
         m_tables.back()->get_sheet_properties();
@@ -550,7 +576,8 @@ void ods_content_xml_context::end_row()
     if (m_row_attr.number_rows_repeated > 1)
     {
         // TODO: repeat this row.
-//      cout << "repeat this row " << m_row_attr.number_rows_repeated << " times" << endl;
+        if (get_config().debug)
+            cout << "TODO: repeat this row " << m_row_attr.number_rows_repeated << " times." << endl;
     }
     m_row += m_row_attr.number_rows_repeated;
 }
@@ -564,7 +591,7 @@ void ods_content_xml_context::start_cell(const xml_attrs_t& attrs)
 void ods_content_xml_context::end_cell()
 {
     name2id_type::const_iterator it = m_cell_format_map.find(m_cell_attr.style_name);
-    if (it != m_cell_format_map.end())
+    if (m_tables.back() && it != m_cell_format_map.end())
         m_tables.back()->set_format(m_row, m_col, it->second);
 
     push_cell_value();
@@ -582,6 +609,7 @@ void ods_content_xml_context::end_cell()
 void ods_content_xml_context::push_cell_value()
 {
     spreadsheet::iface::import_sheet* sheet = m_tables.back();
+
     bool has_formula = !m_cell_attr.formula.empty();
     if (has_formula)
     {
@@ -612,24 +640,27 @@ void ods_content_xml_context::push_cell_value()
         return;
     }
 
-    switch (m_cell_attr.type)
+    if (sheet)
     {
-        case vt_float:
-            sheet->set_value(m_row, m_col, m_cell_attr.value);
-        break;
-        case vt_string:
-            if (m_has_content)
-                sheet->set_string(m_row, m_col, m_para_index);
-        break;
-        case vt_date:
+        switch (m_cell_attr.type)
         {
-            date_time_t val = to_date_time(m_cell_attr.date_value);
-            sheet->set_date_time(
-                m_row, m_col, val.year, val.month, val.day, val.hour, val.minute, val.second);
+            case vt_float:
+                sheet->set_value(m_row, m_col, m_cell_attr.value);
+            break;
+            case vt_string:
+                if (m_has_content)
+                    sheet->set_string(m_row, m_col, m_para_index);
+            break;
+            case vt_date:
+            {
+                date_time_t val = to_date_time(m_cell_attr.date_value);
+                sheet->set_date_time(
+                    m_row, m_col, val.year, val.month, val.day, val.hour, val.minute, val.second);
+            }
+            break;
+            default:
+                ;
         }
-        break;
-        default:
-            ;
     }
 }
 
@@ -650,20 +681,28 @@ void ods_content_xml_context::end_spreadsheet()
             continue;
 
         spreadsheet::iface::import_sheet* sheet = m_tables[data.sheet];
-        sheet->set_formula(
-            data.row, data.column, data.grammar,
-            data.exp.get(), data.exp.size());
-
-        switch (data.result.type)
+        if (sheet)
         {
-            case ods_session_data::rt_numeric:
-                sheet->set_formula_result(data.row, data.column, data.result.numeric_value);
-            break;
-            case ods_session_data::rt_string:
-            case ods_session_data::rt_error:
-            case ods_session_data::rt_none:
-            default:
-                ;
+            spreadsheet::iface::import_formula* formula = sheet->get_formula();
+            if (formula)
+            {
+                formula->set_position(data.row, data.column);
+                formula->set_formula(data.grammar, data.exp.data(), data.exp.size());
+
+                switch (data.result.type)
+                {
+                    case ods_session_data::rt_numeric:
+                        formula->set_result_value(data.result.numeric_value);
+                        break;
+                    case ods_session_data::rt_string:
+                    case ods_session_data::rt_error:
+                    case ods_session_data::rt_none:
+                    default:
+                        ;
+                }
+
+                formula->commit();
+            }
         }
     }
 
