@@ -12,6 +12,7 @@
 #include "orcus/spreadsheet/export_interface.hpp"
 #include "orcus/xml_namespace.hpp"
 #include "orcus/stream.hpp"
+#include "orcus/string_pool.hpp"
 
 #include "xml_map_tree.hpp"
 
@@ -36,38 +37,38 @@ class xml_data_sax_handler
     {
         xmlns_id_t ns;
         pstring name;
-        const char* element_open_begin;
-        const char* element_open_end;
+        std::ptrdiff_t element_open_begin;
+        std::ptrdiff_t element_open_end;
 
         xml_map_tree::element_type type;
 
         scope(xmlns_id_t _ns, const pstring& _name) :
             ns(_ns), name(_name),
-            element_open_begin(nullptr), element_open_end(nullptr),
+            element_open_begin(0), element_open_end(0),
             type(xml_map_tree::element_unknown) {}
     };
 
     vector<sax_ns_parser_attribute> m_attrs;
     vector<scope> m_scopes;
 
+    string_pool m_pool;
     spreadsheet::iface::import_factory& m_factory;
     xml_map_tree::const_element_list_type& m_link_positions;
     const xml_map_tree& m_map_tree;
     xml_map_tree::walker m_map_tree_walker;
 
     const xml_map_tree::element* mp_current_elem;
-
+    pstring m_current_chars;
     bool m_in_range_ref:1;
 
 private:
 
     const sax_ns_parser_attribute* find_attr_by_name(xmlns_id_t ns, const pstring& name)
     {
-        vector<sax_ns_parser_attribute>::const_iterator it = m_attrs.begin(), it_end = m_attrs.end();
-        for (; it != it_end; ++it)
+        for (const sax_ns_parser_attribute& attr : m_attrs)
         {
-            if (it->ns == ns && it->name == name)
-                return &(*it);
+            if (attr.ns == ns && attr.name == name)
+                return &attr;
         }
         return nullptr;
     }
@@ -84,9 +85,6 @@ private:
         assert(field.ref);
         assert(!field.ref->pos.sheet.empty());
 
-        if (field.column_pos == 0)
-            ++field.ref->row_size;
-
         const xml_map_tree::cell_position& pos = field.ref->pos;
         spreadsheet::iface::import_sheet* sheet = m_factory.get_sheet(pos.sheet.get(), pos.sheet.size());
         if (sheet)
@@ -127,17 +125,16 @@ public:
         scope& cur = m_scopes.back();
         cur.element_open_begin = elem.begin_pos;
         cur.element_open_end = elem.end_pos;
+        m_current_chars.clear();
 
         mp_current_elem = m_map_tree_walker.push_element(elem.ns, elem.name);
         if (mp_current_elem)
         {
             // Go through all linked attributes that belong to this element,
             // and see if they exist in this content xml.
-            const xml_map_tree::attribute_store_type& linked_attrs = mp_current_elem->attributes;
-            xml_map_tree::attribute_store_type::const_iterator it = linked_attrs.begin(), it_end = linked_attrs.end();
-            for (; it != it_end; ++it)
+            for (const auto& p_attr : mp_current_elem->attributes)
             {
-                const xml_map_tree::attribute& linked_attr = **it;
+                const xml_map_tree::attribute& linked_attr = *p_attr;
                 const sax_ns_parser_attribute* p = find_attr_by_name(linked_attr.ns, linked_attr.name);
                 if (!p)
                     continue;
@@ -149,10 +146,15 @@ public:
                 {
                     case xml_map_tree::reference_cell:
                         set_single_link_cell(*linked_attr.cell_ref, val_trimmed);
-                    break;
+                        break;
                     case xml_map_tree::reference_range_field:
+                    {
+                        const xml_map_tree::field_in_range& field = *linked_attr.field_ref;
+                        if (field.column_pos == 0)
+                            ++field.ref->row_size;
                         set_field_link_cell(*linked_attr.field_ref, val_trimmed);
-                    break;
+                        break;
+                    }
                     default:
                         ;
                 }
@@ -173,6 +175,25 @@ public:
 
         if (mp_current_elem)
         {
+            switch (mp_current_elem->ref_type)
+            {
+                case xml_map_tree::reference_cell:
+                {
+                    set_single_link_cell(*mp_current_elem->cell_ref, m_current_chars);
+                    break;
+                }
+                case xml_map_tree::reference_range_field:
+                {
+                    const xml_map_tree::field_in_range& field = *mp_current_elem->field_ref;
+                    if (field.column_pos == 0)
+                        ++field.ref->row_size;
+                    set_field_link_cell(*mp_current_elem->field_ref, m_current_chars);
+                    break;
+                }
+                default:
+                    ;
+            }
+
             // Store the end element position in stream for linked elements.
             const scope& cur = m_scopes.back();
             if (mp_current_elem->ref_type == xml_map_tree::reference_cell ||
@@ -199,26 +220,14 @@ public:
         mp_current_elem = m_map_tree_walker.pop_element(elem.ns, elem.name);
     }
 
-    void characters(const pstring& val, bool /*transient*/)
+    void characters(const pstring& val, bool transient)
     {
         if (!mp_current_elem)
             return;
 
-        pstring val_trimmed = val.trim();
-        if (val_trimmed.empty())
-            return;
-
-        switch (mp_current_elem->ref_type)
-        {
-            case xml_map_tree::reference_cell:
-                set_single_link_cell(*mp_current_elem->cell_ref, val_trimmed);
-            break;
-            case xml_map_tree::reference_range_field:
-                set_field_link_cell(*mp_current_elem->field_ref, val_trimmed);
-            break;
-            default:
-                ;
-        }
+        m_current_chars = val.trim();
+        if (transient)
+            m_current_chars = m_pool.intern(m_current_chars).first;
     }
 
     void attribute(const pstring& /*name*/, const pstring& /*val*/)
@@ -273,10 +282,9 @@ void write_opening_element(
 
     os << '<' << elem;
 
-    xml_map_tree::attribute_store_type::const_iterator it = elem.attributes.begin(), it_end = elem.attributes.end();
-    for (; it != it_end; ++it)
+    for (const auto& p_attr : elem.attributes)
     {
-        const xml_map_tree::attribute& attr = **it;
+        const xml_map_tree::attribute& attr = *p_attr;
         if (attr.ref_type != xml_map_tree::reference_range_field)
             // In theory this should never happen but it won't hurt to check.
             continue;
@@ -296,10 +304,9 @@ void write_opening_element(
     ostream& os, const xml_map_tree::element& elem, const spreadsheet::iface::export_factory& fact, bool self_close)
 {
     os << '<' << elem;
-    xml_map_tree::attribute_store_type::const_iterator it = elem.attributes.begin(), it_end = elem.attributes.end();
-    for (; it != it_end; ++it)
+    for (const auto& p_attr : elem.attributes)
     {
-        const xml_map_tree::attribute& attr = **it;
+        const xml_map_tree::attribute& attr = *p_attr;
         if (attr.ref_type != xml_map_tree::reference_cell)
             // We should only see single linked cell here, as all
             // field links are handled by the range parent above.
@@ -448,9 +455,6 @@ struct orcus_xml_impl
     spreadsheet::iface::import_factory* mp_import_factory;
     spreadsheet::iface::export_factory* mp_export_factory;
 
-    /** original xml data stream. */
-    string m_data_strm;
-
     /** xml namespace repository for the whole session. */
     xmlns_repository& m_ns_repo;
 
@@ -460,6 +464,8 @@ struct orcus_xml_impl
     /** xml element tree that represents all mapped paths. */
     xml_map_tree m_map_tree;
 
+    spreadsheet::sheet_t m_sheet_count;
+
     /**
      * Positions of all linked elements, single and range reference alike.
      * Stored link elements must be sorted in order of stream positions, and
@@ -470,7 +476,13 @@ struct orcus_xml_impl
 
     xml_map_tree::cell_position m_cur_range_ref;
 
-    explicit orcus_xml_impl(xmlns_repository& ns_repo) : mp_import_factory(nullptr), mp_export_factory(nullptr), m_ns_repo(ns_repo), m_ns_cxt_map(ns_repo.create_context()), m_map_tree(m_ns_repo) {}
+    explicit orcus_xml_impl(xmlns_repository& ns_repo) :
+        mp_import_factory(nullptr),
+        mp_export_factory(nullptr),
+        m_ns_repo(ns_repo),
+        m_ns_cxt_map(ns_repo.create_context()),
+        m_map_tree(m_ns_repo),
+        m_sheet_count(0) {}
 };
 
 orcus_xml::orcus_xml(xmlns_repository& ns_repo, spreadsheet::iface::import_factory* im_fact, spreadsheet::iface::export_factory* ex_fact) :
@@ -519,26 +531,30 @@ void orcus_xml::append_sheet(const pstring& name)
     if (name.empty())
         return;
 
-    mp_impl->mp_import_factory->append_sheet(name.get(), name.size());
+    mp_impl->mp_import_factory->append_sheet(mp_impl->m_sheet_count++, name.get(), name.size());
 }
 
-void orcus_xml::read_file(const char* filepath)
+void orcus_xml::read_stream(const char* p, size_t n)
 {
 #if ORCUS_DEBUG_XML
     cout << "reading file " << filepath << endl;
 #endif
-    string& strm = mp_impl->m_data_strm;
-    strm = load_file_content(filepath);
+
+    pstring strm(p, n);
+    read_impl(strm);
+}
+
+void orcus_xml::read_impl(const pstring& strm)
+{
     if (strm.empty())
         return;
 
     // Insert the range headers and reset the row size counters.
     xml_map_tree::range_ref_map_type& range_refs = mp_impl->m_map_tree.get_range_references();
-    xml_map_tree::range_ref_map_type::iterator it_ref = range_refs.begin(), it_ref_end = range_refs.end();
-    for (; it_ref != it_ref_end; ++it_ref)
+    for (const auto& ref_pair : range_refs)
     {
-        const xml_map_tree::cell_position& ref = it_ref->first;
-        xml_map_tree::range_reference& range_ref = *it_ref->second;
+        const xml_map_tree::cell_position& ref = ref_pair.first;
+        xml_map_tree::range_reference& range_ref = *ref_pair.second;
         range_ref.row_size = 0; // Reset the row offset.
 
         spreadsheet::iface::import_sheet* sheet =
@@ -568,7 +584,7 @@ void orcus_xml::read_file(const char* filepath)
     xml_data_sax_handler handler(
        *mp_impl->mp_import_factory, mp_impl->m_link_positions, mp_impl->m_map_tree);
 
-    sax_ns_parser<xml_data_sax_handler> parser(strm.c_str(), strm.size(), ns_cxt, handler);
+    sax_ns_parser<xml_data_sax_handler> parser(strm.data(), strm.size(), ns_cxt, handler);
     parser.parse();
 }
 
@@ -585,14 +601,14 @@ void dump_links(const xml_map_tree::const_element_list_type& links)
 
 #endif
 
-void orcus_xml::write_file(const char* filepath)
+void orcus_xml::write(const char* p_in, size_t n_in, std::ostream& out) const
 {
     if (!mp_impl->mp_export_factory)
         // We can't export data witout export factory.
         return;
 
-    if (mp_impl->m_data_strm.empty())
-        // Original xml stream is missing.  We need it.
+    if (!n_in)
+        // Source input stream is empty.
         return;
 
     xml_map_tree::const_element_list_type& links = mp_impl->m_link_positions;
@@ -603,14 +619,6 @@ void orcus_xml::write_file(const char* filepath)
     // Sort all link position by opening element positions.
     std::sort(links.begin(), links.end(), less_by_opening_elem_pos());
 
-#if ORCUS_DEBUG_XML
-    cout << "writing to " << filepath << endl;
-#endif
-    ofstream file(filepath);
-
-    if (!file)
-        throw general_error("Failed to create output file.");
-
     spreadsheet::iface::export_factory& fact = *mp_impl->mp_export_factory;
     xml_map_tree::const_element_list_type::const_iterator it = links.begin(), it_end = links.end();
 
@@ -618,7 +626,11 @@ void orcus_xml::write_file(const char* filepath)
     dump_links(links);
 #endif
 
-    const char* begin_pos = &mp_impl->m_data_strm[0];
+    pstring strm(p_in, n_in);
+
+    const char* p0 = p_in;
+    std::ptrdiff_t begin_pos = 0;
+
     for (; it != it_end; ++it)
     {
         const xml_map_tree::element& elem = **it;
@@ -632,16 +644,16 @@ void orcus_xml::write_file(const char* filepath)
             if (!sheet)
                 continue;
 
-            const char* open_begin = elem.stream_pos.open_begin;
-            const char* close_begin = elem.stream_pos.close_begin;
-            const char* close_end = elem.stream_pos.close_end;
+            std::ptrdiff_t open_begin  = elem.stream_pos.open_begin;
+            std::ptrdiff_t close_begin = elem.stream_pos.close_begin;
+            std::ptrdiff_t close_end   = elem.stream_pos.close_end;
 
             assert(open_begin > begin_pos);
-            file << pstring(begin_pos, open_begin-begin_pos); // stream since last linked element.
+            out << pstring(p0+begin_pos, open_begin-begin_pos); // stream since last linked element.
 
-            write_opening_element(file, elem, fact, false);
-            sheet->write_string(file, pos.row, pos.col);
-            file << pstring(close_begin, close_end-close_begin); // closing element.
+            write_opening_element(out, elem, fact, false);
+            sheet->write_string(out, pos.row, pos.col);
+            out << pstring(p0+close_begin, close_end-close_begin); // closing element.
             begin_pos = close_end;
         }
         else if (elem.range_parent)
@@ -655,16 +667,16 @@ void orcus_xml::write_file(const char* filepath)
             if (!sheet)
                 continue;
 
-            const char* open_begin = elem.stream_pos.open_begin;
-            const char* close_begin = elem.stream_pos.close_begin;
-            const char* close_end = elem.stream_pos.close_end;
+            std::ptrdiff_t open_begin  = elem.stream_pos.open_begin;
+            std::ptrdiff_t close_begin = elem.stream_pos.close_begin;
+            std::ptrdiff_t close_end   = elem.stream_pos.close_end;
 
             assert(open_begin > begin_pos);
-            file << pstring(begin_pos, open_begin-begin_pos); // stream since last linked element.
+            out << pstring(p0+begin_pos, open_begin-begin_pos); // stream since last linked element.
 
-            write_opening_element(file, elem, fact, false);
-            write_range_reference(file, elem, fact);
-            file << pstring(close_begin, close_end-close_begin); // closing element.
+            write_opening_element(out, elem, fact, false);
+            write_range_reference(out, elem, fact);
+            out << pstring(p0+close_begin, close_end-close_begin); // closing element.
             begin_pos = close_end;
         }
         else if (elem.unlinked_attribute_anchor())
@@ -672,15 +684,15 @@ void orcus_xml::write_file(const char* filepath)
             // Element is not linked but has one or more attributes that are
             // linked.  Here, only write the opening element with attributes.
 
-            const char* open_begin = elem.stream_pos.open_begin;
-            const char* open_end = elem.stream_pos.open_end;
+            std::ptrdiff_t open_begin = elem.stream_pos.open_begin;
+            std::ptrdiff_t open_end   = elem.stream_pos.open_end;
 
-            bool self_close = open_begin == elem.stream_pos.close_begin;
+            bool self_close = elem.stream_pos.open_begin == elem.stream_pos.close_begin;
 
             assert(open_begin > begin_pos);
-            file << pstring(begin_pos, open_begin-begin_pos); // stream since last linked element.
+            out << pstring(p0+begin_pos, open_begin-begin_pos); // stream since last linked element.
 
-            write_opening_element(file, elem, fact, self_close);
+            write_opening_element(out, elem, fact, self_close);
             begin_pos = open_end;
         }
         else
@@ -688,9 +700,9 @@ void orcus_xml::write_file(const char* filepath)
     }
 
     // Flush the remaining stream.
-    const char* strm_end = &mp_impl->m_data_strm[mp_impl->m_data_strm.size()-1];
-    file << pstring(begin_pos, strm_end-begin_pos);
+    out << pstring(p0+begin_pos, strm.size()-begin_pos);
 }
 
 }
+
 /* vim:set shiftwidth=4 softtabstop=4 expandtab: */
