@@ -127,6 +127,54 @@ bool operator!= (const range_t& left, const range_t& right)
     return !operator== (left, right);
 }
 
+bool operator< (const range_t& left, const range_t& right)
+{
+    if (left.first.row != right.first.row)
+        return left.first.row < right.first.row;
+
+    if (left.first.column != right.first.column)
+        return left.first.column < right.first.column;
+
+    if (left.last.row != right.last.row)
+        return left.last.row < right.last.row;
+
+    return left.last.column < right.last.column;
+}
+
+bool operator> (const range_t& left, const range_t& right)
+{
+    if (left.first.row != right.first.row)
+        return left.first.row > right.first.row;
+
+    if (left.first.column != right.first.column)
+        return left.first.column > right.first.column;
+
+    if (left.last.row != right.last.row)
+        return left.last.row > right.last.row;
+
+    return left.last.column > right.last.column;
+}
+
+range_t& operator+= (range_t& left, const address_t& right)
+{
+    left.first.column += right.column;
+    left.first.row    += right.row;
+    left.last.column  += right.column;
+    left.last.row     += right.row;
+
+    return left;
+}
+
+range_t& operator-= (range_t& left, const address_t& right)
+{
+    left.first.column -= right.column;
+    left.first.row    -= right.row;
+    left.last.column  -= right.column;
+    left.last.row     -= right.row;
+
+    return left;
+}
+
 std::ostream& operator<< (std::ostream& os, const address_t& v)
 {
     os << "(column=" << v.column << ",row=" << v.row << ")";
