@@ -8,6 +8,7 @@
 #include "xls_xml_context.hpp"
 #include "xls_xml_namespace_types.hpp"
 #include "xls_xml_token_constants.hpp"
+#include "spreadsheet_iface_util.hpp"
 #include "orcus/spreadsheet/import_interface.hpp"
 #include "orcus/spreadsheet/import_interface_view.hpp"
 #include "orcus/measurement.hpp"
@@ -50,11 +51,9 @@ xls_xml_data_context::string_segment_type::string_segment_type(const pstring& _s
     str(_str) {}
 
 xls_xml_data_context::xls_xml_data_context(
-    session_context& session_cxt, const tokens& tokens, spreadsheet::iface::import_factory* factory) :
+    session_context& session_cxt, const tokens& tokens, xls_xml_context& parent_cxt) :
     xml_context_base(session_cxt, tokens),
-    mp_factory(factory),
-    mp_cur_sheet(nullptr),
-    m_row(0), m_col(0),
+    m_parent_cxt(parent_cxt),
     m_cell_type(ct_unknown),
     m_cell_value(std::numeric_limits<double>::quiet_NaN())
 {
@@ -214,10 +213,7 @@ bool xls_xml_data_context::end_element(xmlns_id_t ns, xml_token_t name)
     return pop_stack(ns, name);
 }
 
-void xls_xml_data_context::reset(
-    spreadsheet::iface::import_sheet* sheet,
-    spreadsheet::row_t row, spreadsheet::col_t col,
-    const pstring& cell_formula)
+void xls_xml_data_context::reset()
 {
     m_format_stack.clear();
     m_format_stack.emplace_back(); // set default format.
@@ -226,10 +222,6 @@ void xls_xml_data_context::reset(
     m_cell_type = ct_unknown;
     m_cell_string.clear();
 
-    mp_cur_sheet = sheet;
-    m_row = row;
-    m_col = col;
-    m_cell_formula = cell_formula;
     m_cell_value = std::numeric_limits<double>::quiet_NaN();
     m_cell_datetime = date_time_t();
 }
@@ -268,23 +260,39 @@ void xls_xml_data_context::start_element_data(
 
 void xls_xml_data_context::end_element_data()
 {
-    if (!m_cell_formula.empty())
+    pstring formula = m_parent_cxt.pop_and_clear_formula();
+
+    if (!formula.empty())
     {
-        push_formula_cell();
+        if (m_parent_cxt.is_array_formula())
+            push_array_formula_parent_cell(formula);
+        else
+            push_formula_cell(formula);
         m_cell_type = ct_unknown;
         return;
     }
 
+    if (handle_array_formula_result())
+    {
+        m_cell_type = ct_unknown;
+        return;
+    }
+
+    spreadsheet::iface::import_sheet* sheet = m_parent_cxt.get_import_sheet();
+    spreadsheet::address_t pos = m_parent_cxt.get_current_pos();
+
     switch (m_cell_type)
     {
         case ct_unknown:
             break;
         case ct_number:
-            mp_cur_sheet->set_value(m_row, m_col, m_cell_value);
+            sheet->set_value(pos.row, pos.column, m_cell_value);
             break;
         case ct_string:
         {
-            spreadsheet::iface::import_shared_strings* ss = mp_factory->get_shared_strings();
+            spreadsheet::iface::import_shared_strings* ss =
+                m_parent_cxt.get_import_factory()->get_shared_strings();
+
             if (!ss)
                 break;
 
@@ -295,7 +303,7 @@ void xls_xml_data_context::end_element_data()
             {
                 // Unformatted string.
                 const pstring& s = m_cell_string.back().str;
-                mp_cur_sheet->set_string(m_row, m_col, ss->append(s.data(), s.size()));
+                sheet->set_string(pos.row, pos.column, ss->append(s.data(), s.size()));
             }
             else
             {
@@ -317,7 +325,7 @@ void xls_xml_data_context::end_element_data()
                 }
 
                 size_t si = ss->commit_segments();
-                mp_cur_sheet->set_string(m_row, m_col, si);
+                sheet->set_string(pos.row, pos.column, si);
             }
 
             m_cell_string.clear();
@@ -326,8 +334,8 @@ void xls_xml_data_context::end_element_data()
         }
         case ct_datetime:
         {
-            mp_cur_sheet->set_date_time(
-                m_row, m_col,
+            sheet->set_date_time(
+                pos.row, pos.column,
                 m_cell_datetime.year, m_cell_datetime.month, m_cell_datetime.day,
                 m_cell_datetime.hour, m_cell_datetime.minute, m_cell_datetime.second);
             break;
@@ -340,22 +348,126 @@ void xls_xml_data_context::end_element_data()
     m_cell_type = ct_unknown;
 }
 
-void xls_xml_data_context::push_formula_cell()
+bool xls_xml_data_context::handle_array_formula_result()
 {
-    mp_cur_sheet->set_formula(
-        m_row, m_col, spreadsheet::formula_grammar_t::xls_xml,
-        m_cell_formula.get(), m_cell_formula.size());
+    xls_xml_context::array_formulas_type& store = m_parent_cxt.get_array_formula_store();
+    spreadsheet::address_t cur_pos = m_parent_cxt.get_current_pos();
+
+    // See if the current cell is within an array formula range.
+    auto it = store.begin(), ite = store.end();
+
+    while (it != ite)
+    {
+        const spreadsheet::range_t& ref = it->first;
+        xls_xml_context::array_formula_type& af = *it->second;
+
+        if (ref.last.row < cur_pos.row)
+        {
+            // If this result range lies above the current row, push the array
+            // and delete it from the list.
+
+            spreadsheet::iface::import_sheet* sheet = m_parent_cxt.get_import_sheet();
+            spreadsheet::iface::import_array_formula* array = nullptr;
+
+            if (sheet)
+                array = sheet->get_array_formula();
+
+            if (array)
+            {
+                push_array_formula(
+                    array, ref, af.formula, spreadsheet::formula_grammar_t::xls_xml, af.results);
+            }
+
+            store.erase(it++);
+            continue;
+        }
+
+        if (cur_pos.column < ref.first.column || ref.last.column < cur_pos.column ||
+            cur_pos.row < ref.first.row || ref.last.row < cur_pos.row)
+        {
+            // This cell is not within this array formula range.  Move on to
+            // the next one.
+            ++it;
+            continue;
+        }
+
+        size_t row_offset = cur_pos.row - ref.first.row;
+        size_t col_offset = cur_pos.column - ref.first.column;
+        range_formula_results& res = af.results;
+        push_array_result(res, row_offset, col_offset);
+
+        return true;
+    }
+
+    return false;
+}
+
+void xls_xml_data_context::push_array_result(
+    range_formula_results& res, size_t row_offset, size_t col_offset)
+{
+    switch (m_cell_type)
+    {
+        case ct_number:
+        {
+            res.set(row_offset, col_offset, m_cell_value);
+            break;
+        }
+        case ct_unknown:
+        case ct_datetime:
+        case ct_string:
+        default:
+            if (get_config().debug)
+                cout << "warning: unknown cell type '" << m_cell_type << "': value not pushed." << endl;
+    }
+}
+
+void xls_xml_data_context::push_formula_cell(const pstring& formula)
+{
+    spreadsheet::iface::import_sheet* sheet = m_parent_cxt.get_import_sheet();
+    spreadsheet::address_t pos = m_parent_cxt.get_current_pos();
+    spreadsheet::iface::import_formula* xformula = sheet->get_formula();
+
+    if (xformula)
+    {
+        xformula->set_position(pos.row, pos.column);
+        xformula->set_formula(spreadsheet::formula_grammar_t::xls_xml, formula.data(), formula.size());
+
+        switch (m_cell_type)
+        {
+            case ct_number:
+                xformula->set_result_value(m_cell_value);
+                break;
+            default:
+                ;
+        }
+
+        xformula->commit();
+    }
+}
+
+void xls_xml_data_context::push_array_formula_parent_cell(const pstring& formula)
+{
+    spreadsheet::address_t pos = m_parent_cxt.get_current_pos();
+    spreadsheet::range_t range = m_parent_cxt.get_array_range();
+    xls_xml_context::array_formulas_type& store = m_parent_cxt.get_array_formula_store();
+
+    range += pos;
+
+    store.push_back(
+        std::make_pair(
+            range,
+            orcus::make_unique<xls_xml_context::array_formula_type>(range, formula)));
+
+    xls_xml_context::array_formula_type& af = *store.back().second;
 
     switch (m_cell_type)
     {
         case ct_number:
-            mp_cur_sheet->set_formula_result(m_row, m_col, m_cell_value);
+            af.results.set(0, 0, m_cell_value);
             break;
         default:
             ;
     }
-
-    m_cell_formula.clear();
 }
 
 void xls_xml_data_context::update_current_format()
@@ -494,6 +606,11 @@ const map_type& get()
 
 }
 
+xls_xml_context::array_formula_type::array_formula_type(
+    const spreadsheet::range_t& range, const pstring& formula) :
+    formula(formula),
+    results(range.last.row-range.first.row+1, range.last.column-range.first.column+1) {}
+
 xls_xml_context::named_exp::named_exp(const pstring& _name, const pstring& _expression, spreadsheet::sheet_t _scope) :
     name(_name), expression(_expression), scope(_scope) {}
 
@@ -565,14 +682,27 @@ xls_xml_context::xls_xml_context(session_context& session_cxt, const tokens& tok
     m_cur_row(0), m_cur_col(0),
     m_cur_prop_col(0),
     m_cur_merge_down(0), m_cur_merge_across(0),
-    m_cc_data(session_cxt, tokens, factory)
+    m_active_pane(spreadsheet::sheet_pane_t::unspecified),
+    m_cc_data(session_cxt, tokens, *this)
 {
+    m_cur_array_range.first.column = -1;
+    m_cur_array_range.first.row = -1;
+    m_cur_array_range.last = m_cur_array_range.first;
 }
 
 xls_xml_context::~xls_xml_context()
 {
 }
 
+void xls_xml_context::declaration(const xml_declaration_t& decl)
+{
+    spreadsheet::iface::import_global_settings* gs = mp_factory->get_global_settings();
+    if (!gs)
+        return;
+
+    gs->set_character_set(decl.encoding);
+}
+
 bool xls_xml_context::can_handle_element(xmlns_id_t ns, xml_token_t name) const
 {
     if (ns == NS_xls_xml_ss)
@@ -597,8 +727,8 @@ xml_context_base* xls_xml_context::create_child_context(xmlns_id_t ns, xml_token
             case XML_Data:
             {
                 // Move the cell formula string to the Data element context.
-                m_cc_data.reset(mp_cur_sheet, m_cur_row, m_cur_col, m_cur_cell_formula);
-                m_cur_cell_formula.clear();
+                m_cc_data.transfer_common(*this);
+                m_cc_data.reset();
                 return &m_cc_data;
             }
             default:
@@ -940,6 +1070,9 @@ bool xls_xml_context::end_element(xmlns_id_t ns, xml_token_t name)
             case XML_Column:
                 end_element_column();
                 break;
+            case XML_Table:
+                end_element_table();
+                break;
             case XML_Workbook:
                 end_element_workbook();
                 break;
@@ -1164,43 +1297,53 @@ void xls_xml_context::start_element_cell(const xml_token_pair_t& parent, const x
     m_cur_merge_across = 0; // extra column(s) that are part of the merged cell.
     m_cur_merge_down = 0; // extra row(s) that are part of the merged cell.
 
-    std::for_each(attrs.begin(), attrs.end(),
-        [&](const xml_token_attr_t& attr)
+    m_cur_array_range.first.column = -1;
+    m_cur_array_range.first.row = -1;
+    m_cur_array_range.last = m_cur_array_range.first;
+
+    for (const xml_token_attr_t& attr : attrs)
+    {
+        if (attr.value.empty())
+            return;
+
+        if (attr.ns != NS_xls_xml_ss)
+            return;
+
+        switch (attr.name)
         {
-            if (attr.value.empty())
-                return;
-
-            if (attr.ns != NS_xls_xml_ss)
-                return;
-
-            switch (attr.name)
+            case XML_Index:
+                col_index = to_long(attr.value);
+                break;
+            case XML_Formula:
+                if (attr.value[0] == '=' && attr.value.size() > 1)
+                {
+                    pstring s(attr.value.get()+1, attr.value.size()-1);
+                    formula = s;
+                    if (attr.transient)
+                        formula = intern(s);
+                }
+                break;
+            case XML_MergeAcross:
+                m_cur_merge_across = to_long(attr.value);
+                break;
+            case XML_MergeDown:
+                m_cur_merge_down = to_long(attr.value);
+                break;
+            case XML_StyleID:
+                m_cur_cell_style_id = intern(attr);
+                break;
+            case XML_ArrayRange:
             {
-                case XML_Index:
-                    col_index = to_long(attr.value);
-                    break;
-                case XML_Formula:
-                    if (attr.value[0] == '=' && attr.value.size() > 1)
-                    {
-                        pstring s(attr.value.get()+1, attr.value.size()-1);
-                        formula = s;
-                        if (attr.transient)
-                            formula = intern(s);
-                    }
-                    break;
-                case XML_MergeAcross:
-                    m_cur_merge_across = to_long(attr.value);
-                    break;
-                case XML_MergeDown:
-                    m_cur_merge_down = to_long(attr.value);
-                    break;
-                case XML_StyleID:
-                    m_cur_cell_style_id = intern(attr);
-                    break;
-                default:
-                    ;
+                spreadsheet::iface::import_reference_resolver* resolver = mp_factory->get_reference_resolver();
+                if (resolver)
+                    m_cur_array_range = resolver->resolve_range(attr.value.data(), attr.value.size());
+
+                break;
             }
+            default:
+                ;
         }
-    );
+    }
 
     if (!formula.empty())
         m_cur_cell_formula = formula;
@@ -1347,9 +1490,16 @@ void xls_xml_context::end_element_cell()
     if (mp_cur_sheet && !m_cur_cell_formula.empty())
     {
         // Likely a Cell element without a child Data element.
-        mp_cur_sheet->set_formula(
-            m_cur_row, m_cur_col, spreadsheet::formula_grammar_t::xls_xml,
-            m_cur_cell_formula.data(), m_cur_cell_formula.size());
+        spreadsheet::iface::import_formula* xformula = mp_cur_sheet->get_formula();
+
+        if (xformula)
+        {
+            xformula->set_position(m_cur_row, m_cur_col);
+            xformula->set_formula(
+                spreadsheet::formula_grammar_t::xls_xml,
+                m_cur_cell_formula.data(), m_cur_cell_formula.size());
+            xformula->commit();
+        }
     }
 
     m_cur_cell_formula.clear();
@@ -1368,6 +1518,12 @@ void xls_xml_context::end_element_row()
     ++m_cur_row;
 }
 
+void xls_xml_context::end_element_table()
+{
+    push_all_array_formulas();
+    m_array_formulas.clear();
+}
+
 void xls_xml_context::end_element_workbook()
 {
     spreadsheet::iface::import_named_expression* ne_global = mp_factory->get_named_expression();
@@ -1543,7 +1699,7 @@ void xls_xml_context::commit_styles()
         if (style->fill.solid)
         {
             // TODO : add support for fill types other than 'solid'.
-            styles->set_fill_pattern_type(ORCUS_ASCII("solid"));
+            styles->set_fill_pattern_type(spreadsheet::fill_pattern_t::solid);
             styles->set_fill_fg_color(255,
                 style->fill.color.red,
                 style->fill.color.green,
@@ -1583,6 +1739,73 @@ void xls_xml_context::commit_styles()
     }
 }
 
+void xls_xml_context::push_all_array_formulas()
+{
+    if (!mp_cur_sheet)
+        return;
+
+    spreadsheet::iface::import_array_formula* array = mp_cur_sheet->get_array_formula();
+    if (!array)
+        return;
+
+    for (const array_formula_pair_type& pair : m_array_formulas)
+    {
+        const array_formula_type& af = *pair.second;
+        push_array_formula(
+            array, pair.first, af.formula, spreadsheet::formula_grammar_t::xls_xml, af.results);
+    }
+}
+
+spreadsheet::iface::import_factory* xls_xml_context::get_import_factory()
+{
+    return mp_factory;
+}
+
+spreadsheet::iface::import_sheet* xls_xml_context::get_import_sheet()
+{
+    return mp_cur_sheet;
+}
+
+spreadsheet::address_t xls_xml_context::get_current_pos() const
+{
+    spreadsheet::address_t pos;
+    pos.row = m_cur_row;
+    pos.column = m_cur_col;
+    return pos;
+}
+
+pstring xls_xml_context::pop_and_clear_formula()
+{
+    pstring f = m_cur_cell_formula;
+    m_cur_cell_formula.clear();
+    return f;
+}
+
+bool xls_xml_context::is_array_formula() const
+{
+    if (m_cur_array_range.first.column < 0 || m_cur_array_range.first.row < 0)
+        return false;
+
+    if (m_cur_array_range.last.column < 0 || m_cur_array_range.last.row < 0)
+        return false;
+
+    if (m_cur_array_range.first.column > m_cur_array_range.last.column ||
+        m_cur_array_range.first.row > m_cur_array_range.last.row)
+        return false;
+
+    return true;
+}
+
+const spreadsheet::range_t& xls_xml_context::get_array_range() const
+{
+    return m_cur_array_range;
+}
+
+xls_xml_context::array_formulas_type& xls_xml_context::get_array_formula_store()
+{
+    return m_array_formulas;
+}
+
 }
 
 /* vim:set shiftwidth=4 softtabstop=4 expandtab: */
