@@ -240,6 +240,40 @@ void xlsx_shared_strings_context::characters(const pstring& str, bool transient)
     if (cur_token.first == NS_ooxml_xlsx && cur_token.second == XML_t)
     {
         m_cur_str = str;
+
+        // In case the string contains carriage returns (CRs), remove them.
+        m_cell_buffer.reset();
+        const char* p = m_cur_str.data();
+        const char* p_end = p + m_cur_str.size();
+        const char* p0 = nullptr;
+
+        for (; p != p_end; ++p)
+        {
+            if (!p0)
+                p0 = p;
+
+            if (*p == 0x0D)
+            {
+                // Append the segment up to this CR, and skip the CR.
+                m_cell_buffer.append(p0, std::distance(p0, p));
+                p0 = nullptr;
+            }
+        }
+
+        if (!m_cell_buffer.empty())
+        {
+            // This string contains at least one CR.
+
+            if (p0)
+                // Append the tail end.
+                m_cell_buffer.append(p0, std::distance(p0, p));
+
+            m_cur_str = m_pool.intern(
+                m_cell_buffer.get(), m_cell_buffer.size()).first;
+
+            transient = false;
+        }
+
         if (transient)
             m_cur_str = m_pool.intern(m_cur_str).first;
     }
@@ -249,26 +283,73 @@ void xlsx_shared_strings_context::characters(const pstring& str, bool transient)
 
 namespace {
 
-typedef mdds::sorted_string_map<spreadsheet::border_style_t> border_style_map;
+namespace border_style {
 
-border_style_map::entry border_style_type_entries[] =
+typedef mdds::sorted_string_map<spreadsheet::border_style_t> map_type;
+
+// Keys must be sorted.
+const std::vector<map_type::entry> entries =
 {
-    {MDDS_ASCII("dashDot"), spreadsheet::border_style_t::dash_dot},
-    {MDDS_ASCII("dashDotDot"), spreadsheet::border_style_t::dash_dot_dot},
-    {MDDS_ASCII("dashed"), spreadsheet::border_style_t::dashed},
-    {MDDS_ASCII("dotted"), spreadsheet::border_style_t::dotted},
-    {MDDS_ASCII("double"), spreadsheet::border_style_t::double_border},
-    {MDDS_ASCII("hair"), spreadsheet::border_style_t::hair},
-    {MDDS_ASCII("medium"), spreadsheet::border_style_t::medium},
-    {MDDS_ASCII("mediumDashDot"), spreadsheet::border_style_t::medium_dash_dot},
-    {MDDS_ASCII("mediumDashDotDot"), spreadsheet::border_style_t::medium_dash_dot_dot},
-    {MDDS_ASCII("mediumDashed"), spreadsheet::border_style_t::medium_dashed},
-    {MDDS_ASCII("none"), spreadsheet::border_style_t::none},
-    {MDDS_ASCII("slantDashDot"), spreadsheet::border_style_t::slant_dash_dot},
-    {MDDS_ASCII("thick"), spreadsheet::border_style_t::thick},
-    {MDDS_ASCII("thin"), spreadsheet::border_style_t::thin}
+    { ORCUS_ASCII("dashDot"),          spreadsheet::border_style_t::dash_dot            },
+    { ORCUS_ASCII("dashDotDot"),       spreadsheet::border_style_t::dash_dot_dot        },
+    { ORCUS_ASCII("dashed"),           spreadsheet::border_style_t::dashed              },
+    { ORCUS_ASCII("dotted"),           spreadsheet::border_style_t::dotted              },
+    { ORCUS_ASCII("double"),           spreadsheet::border_style_t::double_border       },
+    { ORCUS_ASCII("hair"),             spreadsheet::border_style_t::hair                },
+    { ORCUS_ASCII("medium"),           spreadsheet::border_style_t::medium              },
+    { ORCUS_ASCII("mediumDashDot"),    spreadsheet::border_style_t::medium_dash_dot     },
+    { ORCUS_ASCII("mediumDashDotDot"), spreadsheet::border_style_t::medium_dash_dot_dot },
+    { ORCUS_ASCII("mediumDashed"),     spreadsheet::border_style_t::medium_dashed       },
+    { ORCUS_ASCII("none"),             spreadsheet::border_style_t::none                },
+    { ORCUS_ASCII("slantDashDot"),     spreadsheet::border_style_t::slant_dash_dot      },
+    { ORCUS_ASCII("thick"),            spreadsheet::border_style_t::thick               },
+    { ORCUS_ASCII("thin"),             spreadsheet::border_style_t::thin                }
 };
 
+const map_type& get()
+{
+    static map_type mt(entries.data(), entries.size(), spreadsheet::border_style_t::none);
+    return mt;
+}
+
+}
+
+namespace fill_pattern {
+
+typedef mdds::sorted_string_map<spreadsheet::fill_pattern_t> map_type;
+
+// Keys must be sorted.
+const std::vector<map_type::entry> entries =
+{
+    { ORCUS_ASCII("darkDown"),        spreadsheet::fill_pattern_t::dark_down        },
+    { ORCUS_ASCII("darkGray"),        spreadsheet::fill_pattern_t::dark_gray        },
+    { ORCUS_ASCII("darkGrid"),        spreadsheet::fill_pattern_t::dark_grid        },
+    { ORCUS_ASCII("darkHorizontal"),  spreadsheet::fill_pattern_t::dark_horizontal  },
+    { ORCUS_ASCII("darkTrellis"),     spreadsheet::fill_pattern_t::dark_trellis     },
+    { ORCUS_ASCII("darkUp"),          spreadsheet::fill_pattern_t::dark_up          },
+    { ORCUS_ASCII("darkVertical"),    spreadsheet::fill_pattern_t::dark_vertical    },
+    { ORCUS_ASCII("gray0625"),        spreadsheet::fill_pattern_t::gray_0625        },
+    { ORCUS_ASCII("gray125"),         spreadsheet::fill_pattern_t::gray_125         },
+    { ORCUS_ASCII("lightDown"),       spreadsheet::fill_pattern_t::light_down       },
+    { ORCUS_ASCII("lightGray"),       spreadsheet::fill_pattern_t::light_gray       },
+    { ORCUS_ASCII("lightGrid"),       spreadsheet::fill_pattern_t::light_grid       },
+    { ORCUS_ASCII("lightHorizontal"), spreadsheet::fill_pattern_t::light_horizontal },
+    { ORCUS_ASCII("lightTrellis"),    spreadsheet::fill_pattern_t::light_trellis    },
+    { ORCUS_ASCII("lightUp"),         spreadsheet::fill_pattern_t::light_up         },
+    { ORCUS_ASCII("lightVertical"),   spreadsheet::fill_pattern_t::light_vertical   },
+    { ORCUS_ASCII("mediumGray"),      spreadsheet::fill_pattern_t::medium_gray      },
+    { ORCUS_ASCII("none"),            spreadsheet::fill_pattern_t::none             },
+    { ORCUS_ASCII("solid"),           spreadsheet::fill_pattern_t::solid            },
+};
+
+const map_type& get()
+{
+    static map_type mt(entries.data(), entries.size(), spreadsheet::fill_pattern_t::none);
+    return mt;
+}
+
+}
+
 class border_attr_parser : public unary_function<xml_token_attr_t, void>
 {
     spreadsheet::border_direction_t m_dir;
@@ -283,9 +364,8 @@ public:
         {
             case XML_style:
             {
-                border_style_map type_map(border_style_type_entries, ORCUS_N_ELEMENTS(border_style_type_entries), spreadsheet::border_style_t::none);
                 m_styles.set_border_style(m_dir,
-                        type_map.find(attr.value.get(), attr.value.size()));
+                    border_style::get().find(attr.value.get(), attr.value.size()));
             }
             break;
         }
@@ -612,6 +692,7 @@ void xlsx_styles_context::start_element(xmlns_id_t ns, xml_token_t name, const x
                     case XML_bottom:
                     case XML_left:
                     case XML_right:
+                    case XML_diagonal:
                         // This color is for a border.
                         start_border_color(attrs);
                     break;
@@ -658,7 +739,7 @@ void xlsx_styles_context::start_element(xmlns_id_t ns, xml_token_t name, const x
         {
             xml_element_expected(parent, NS_ooxml_xlsx, XML_fill);
             pstring ps = for_each(attrs.begin(), attrs.end(), single_attr_getter(m_pool, NS_ooxml_xlsx, XML_patternType)).get_value();
-            mp_styles->set_fill_pattern_type(ps.get(), ps.size());
+            mp_styles->set_fill_pattern_type(fill_pattern::get().find(ps.data(), ps.size()));
         }
         break;
         case XML_fgColor:
