@@ -26,14 +26,12 @@
 #include <iostream>
 #endif
 
-namespace orcus {
+namespace orcus { namespace yaml {
 
-yaml_document_error::yaml_document_error(const std::string& msg) :
+document_error::document_error(const std::string& msg) :
     general_error("yaml_document_error", msg) {}
 
-yaml_document_error::~yaml_document_error() throw() {}
-
-namespace yaml { namespace detail {
+document_error::~document_error() throw() {}
 
 struct yaml_value
 {
@@ -92,13 +90,8 @@ struct yaml_value
     };
 };
 
-}}
-
 namespace {
 
-using node_t = yaml::detail::node_t;
-using yaml_value = yaml::detail::yaml_value;
-
 struct yaml_value_string : public yaml_value
 {
     std::string value_string;
@@ -120,7 +113,6 @@ struct yaml_value_number : public yaml_value
 {
     double value_number;
 
-    yaml_value_number() : yaml_value(node_t::number) {}
     yaml_value_number(double num) : yaml_value(node_t::number), value_number(num) {}
     virtual ~yaml_value_number() {}
 
@@ -199,7 +191,6 @@ class handler
                 yvs->value_sequence.push_back(std::move(value));
                 return yvs->value_sequence.back().get();
             }
-            break;
             case node_t::map:
             {
                 yaml_value_map* yvm = static_cast<yaml_value_map*>(cur.node);
@@ -213,16 +204,13 @@ class handler
 
                 return r.first->second.get();
             }
-            break;
             default:
-            {
-                std::ostringstream os;
-                os << BOOST_CURRENT_FUNCTION << ": unstackable YAML value type (" << cur.node->print() << ").";
-                throw yaml::parse_error(os.str());
-            }
+                break;
         }
 
-        return nullptr;
+        std::ostringstream os;
+        os << BOOST_CURRENT_FUNCTION << ": unstackable YAML value type (" << cur.node->print() << ").";
+        throw document_error(os.str());
     }
 
 public:
@@ -382,48 +370,46 @@ public:
     }
 };
 
-}
+} // anonymous namespace
 
-struct yaml_document_tree::impl
+struct document_tree::impl
 {
     std::vector<document_root_type> m_docs;
 };
 
-namespace yaml { namespace detail {
-
-struct node::impl
+struct const_node::impl
 {
     const yaml_value* m_node;
 
     impl(const yaml_value* yv) : m_node(yv) {}
 };
 
-node::node(const yaml_value* yv) : mp_impl(orcus::make_unique<impl>(yv)) {}
-node::node(const node& other) : mp_impl(orcus::make_unique<impl>(other.mp_impl->m_node)) {}
-node::node(node&& rhs) : mp_impl(std::move(rhs.mp_impl)) {}
-node::~node() {}
+const_node::const_node(const yaml_value* yv) : mp_impl(orcus::make_unique<impl>(yv)) {}
+const_node::const_node(const const_node& other) : mp_impl(orcus::make_unique<impl>(other.mp_impl->m_node)) {}
+const_node::const_node(const_node&& rhs) : mp_impl(std::move(rhs.mp_impl)) {}
+const_node::~const_node() {}
 
-node& node::operator=(const node& other)
+const_node& const_node::operator=(const const_node& other)
 {
     if (this == &other)
         return *this;
 
-    node tmp(other);
+    const_node tmp(other);
     mp_impl.swap(tmp.mp_impl);
     return *this;
 }
 
-uintptr_t node::identity() const
+uintptr_t const_node::identity() const
 {
     return reinterpret_cast<uintptr_t>(mp_impl->m_node);
 }
 
-node_t node::type() const
+node_t const_node::type() const
 {
     return mp_impl->m_node->type;
 }
 
-size_t node::child_count() const
+size_t const_node::child_count() const
 {
     switch (mp_impl->m_node->type)
     {
@@ -443,36 +429,36 @@ size_t node::child_count() const
     return 0;
 }
 
-std::vector<node> node::keys() const
+std::vector<const_node> const_node::keys() const
 {
     if (mp_impl->m_node->type != node_t::map)
-        throw yaml_document_error("node::keys: this node is not of map type.");
+        throw document_error("node::keys: this node is not of map type.");
 
     const yaml_value_map* yvm = static_cast<const yaml_value_map*>(mp_impl->m_node);
-    std::vector<node> keys;
+    std::vector<const_node> keys;
     std::for_each(yvm->key_order.begin(), yvm->key_order.end(),
         [&](const std::unique_ptr<yaml_value>& key)
         {
-            keys.push_back(std::move(node(key.get())));
+            keys.push_back(const_node(key.get()));
         }
     );
 
     return keys;
 }
 
-node node::key(size_t index) const
+const_node const_node::key(size_t index) const
 {
     if (mp_impl->m_node->type != node_t::map)
-        throw yaml_document_error("node::key: this node is not of map type.");
+        throw document_error("node::key: this node is not of map type.");
 
     const yaml_value_map* yvm = static_cast<const yaml_value_map*>(mp_impl->m_node);
     if (index >= yvm->key_order.size())
         throw std::out_of_range("node::key: index is out-of-range.");
 
-    return node(yvm->key_order[index].get());
+    return const_node(yvm->key_order[index].get());
 }
 
-node node::child(size_t index) const
+const_node const_node::child(size_t index) const
 {
     switch (mp_impl->m_node->type)
     {
@@ -485,7 +471,7 @@ node node::child(size_t index) const
             const yaml_value* key = yvm->key_order[index].get();
             auto it = yvm->value_map.find(key);
             assert(it != yvm->value_map.end());
-            return node(it->second.get());
+            return const_node(it->second.get());
         }
         break;
         case node_t::sequence:
@@ -494,7 +480,7 @@ node node::child(size_t index) const
             if (index >= yvs->value_sequence.size())
                 throw std::out_of_range("node::child: index is out-of-range");
 
-            return node(yvs->value_sequence[index].get());
+            return const_node(yvs->value_sequence[index].get());
         }
         break;
         case node_t::string:
@@ -504,56 +490,59 @@ node node::child(size_t index) const
         case node_t::null:
         case node_t::unset:
         default:
-            throw yaml_document_error("node::child: this node cannot have child nodes.");
+            throw document_error("node::child: this node cannot have child nodes.");
     }
 }
 
-node node::child(const node& key) const
+const_node const_node::child(const const_node& key) const
 {
     if (mp_impl->m_node->type != node_t::map)
-        throw yaml_document_error("node::child: this node is not of map type.");
+        throw document_error("node::child: this node is not of map type.");
 
     const yaml_value_map* yvm = static_cast<const yaml_value_map*>(mp_impl->m_node);
     auto it = yvm->value_map.find(key.mp_impl->m_node);
     if (it == yvm->value_map.end())
-        throw yaml_document_error("node::child: this map does not have the specified key.");
+        throw document_error("node::child: this map does not have the specified key.");
 
-    return node(it->second.get());
+    return const_node(it->second.get());
 }
 
-node node::parent() const
+const_node const_node::parent() const
 {
     if (!mp_impl->m_node->parent)
-        throw yaml_document_error("node::parent: this node has no parent.");
+        throw document_error("node::parent: this node has no parent.");
 
-    return node(mp_impl->m_node->parent);
+    return const_node(mp_impl->m_node->parent);
 }
 
-pstring node::string_value() const
+pstring const_node::string_value() const
 {
     if (mp_impl->m_node->type != node_t::string)
-        throw yaml_document_error("node::key: current node is not of string type.");
+        throw document_error("node::key: current node is not of string type.");
 
     const yaml_value_string* yvs = static_cast<const yaml_value_string*>(mp_impl->m_node);
     const std::string& str = yvs->value_string;
     return pstring(str.data(), str.size());
 }
 
-double node::numeric_value() const
+double const_node::numeric_value() const
 {
     if (mp_impl->m_node->type != node_t::number)
-        throw yaml_document_error("node::key: current node is not of numeric type.");
+        throw document_error("node::key: current node is not of numeric type.");
 
     const yaml_value_number* yvn = static_cast<const yaml_value_number*>(mp_impl->m_node);
     return yvn->value_number;
 }
 
-}}
+document_tree::document_tree() :
+    mp_impl(orcus::make_unique<impl>()) {}
 
-yaml_document_tree::yaml_document_tree() : mp_impl(orcus::make_unique<impl>()) {}
-yaml_document_tree::~yaml_document_tree() {}
+document_tree::document_tree(document_tree&& other) :
+    mp_impl(std::move(other.mp_impl)) {}
 
-void yaml_document_tree::load(const std::string& strm)
+document_tree::~document_tree() {}
+
+void document_tree::load(const std::string& strm)
 {
     handler hdl;
     yaml_parser<handler> parser(strm.data(), strm.size(), hdl);
@@ -561,14 +550,14 @@ void yaml_document_tree::load(const std::string& strm)
     hdl.swap(mp_impl->m_docs);
 }
 
-size_t yaml_document_tree::get_document_count() const
+size_t document_tree::get_document_count() const
 {
     return mp_impl->m_docs.size();
 }
 
-yaml_document_tree::node yaml_document_tree::get_document_root(size_t index) const
+const_node document_tree::get_document_root(size_t index) const
 {
-    return node(mp_impl->m_docs[index].get());
+    return const_node(mp_impl->m_docs[index].get());
 }
 
 namespace {
@@ -619,34 +608,34 @@ void dump_yaml_node(std::ostringstream& os, const yaml_value& node, size_t scope
 {
     switch (node.type)
     {
-        case yaml_node_t::map:
+        case node_t::map:
             dump_yaml_map(os, node, scope);
         break;
-        case yaml_node_t::sequence:
+        case node_t::sequence:
             dump_yaml_sequence(os, node, scope);
         break;
-        case yaml_node_t::boolean_true:
+        case node_t::boolean_true:
             dump_indent(os, scope);
             os << kw_true << std::endl;
         break;
-        case yaml_node_t::boolean_false:
+        case node_t::boolean_false:
             dump_indent(os, scope);
             os << kw_false << std::endl;
         break;
-        case yaml_node_t::null:
+        case node_t::null:
             dump_indent(os, scope);
             os << kw_tilde << std::endl;
         break;
-        case yaml_node_t::number:
+        case node_t::number:
             dump_indent(os, scope);
             os << static_cast<const yaml_value_number&>(node).value_number << std::endl;
         break;
-        case yaml_node_t::string:
+        case node_t::string:
             dump_indent(os, scope);
             dump_yaml_string(os, static_cast<const yaml_value_string&>(node).value_string);
             os << std::endl;
         break;
-        case yaml_node_t::unset:
+        case node_t::unset:
         default:
             ;
     }
@@ -656,8 +645,8 @@ void dump_yaml_container_item(std::ostringstream& os, const yaml_value& node, si
 {
     switch (node.type)
     {
-        case yaml_node_t::map:
-        case yaml_node_t::sequence:
+        case node_t::map:
+        case node_t::sequence:
             // End the line and dump this child container in the next scope.
             os << std::endl;
             dump_yaml_node(os, node, scope+1);
@@ -680,33 +669,33 @@ void dump_yaml_map(std::ostringstream& os, const yaml_value& node, size_t scope)
 
             switch (key->type)
             {
-                case yaml_node_t::map:
+                case node_t::map:
                     // TODO
                 break;
-                case yaml_node_t::sequence:
+                case node_t::sequence:
                     // TODO
                 break;
-                case yaml_node_t::boolean_true:
+                case node_t::boolean_true:
                     dump_indent(os, scope);
                     os << kw_true;
                 break;
-                case yaml_node_t::boolean_false:
+                case node_t::boolean_false:
                     dump_indent(os, scope);
                     os << kw_false;
                 break;
-                case yaml_node_t::null:
+                case node_t::null:
                     dump_indent(os, scope);
                     os << kw_tilde;
                 break;
-                case yaml_node_t::number:
+                case node_t::number:
                     dump_indent(os, scope);
                     os << static_cast<const yaml_value_number*>(key)->value_number;
                 break;
-                case yaml_node_t::string:
+                case node_t::string:
                     dump_indent(os, scope);
                     dump_yaml_string(os, static_cast<const yaml_value_string*>(key)->value_string);
                 break;
-                case yaml_node_t::unset:
+                case node_t::unset:
                 default:
                     ;
             }
@@ -764,7 +753,7 @@ void dump_json_node(std::ostringstream& os, const yaml_value& node, size_t scope
 
     switch (node.type)
     {
-        case yaml_node_t::map:
+        case node_t::map:
         {
             auto& key_order = static_cast<const yaml_value_map&>(node).key_order;
             auto& vals = static_cast<const yaml_value_map&>(node).value_map;
@@ -776,8 +765,8 @@ void dump_json_node(std::ostringstream& os, const yaml_value& node, size_t scope
             for (auto it = key_order.begin(), ite = key_order.end(); it != ite; ++it, ++pos)
             {
                 const yaml_value* key = it->get();
-                if (key->type != yaml_node_t::string)
-                    throw yaml_document_error("JSON doesn't support non-string key.");
+                if (key->type != node_t::string)
+                    throw document_error("JSON doesn't support non-string key.");
 
                 auto val_pos = vals.find(key);
                 assert(val_pos != vals.end());
@@ -792,7 +781,7 @@ void dump_json_node(std::ostringstream& os, const yaml_value& node, size_t scope
             os << "}";
         }
         break;
-        case yaml_node_t::sequence:
+        case node_t::sequence:
         {
             auto& vals = static_cast<const yaml_value_sequence&>(node).value_sequence;
             os << "[" << std::endl;
@@ -805,22 +794,22 @@ void dump_json_node(std::ostringstream& os, const yaml_value& node, size_t scope
             os << "]";
         }
         break;
-        case yaml_node_t::boolean_true:
+        case node_t::boolean_true:
             os << kw_true;
         break;
-        case yaml_node_t::boolean_false:
+        case node_t::boolean_false:
             os << kw_false;
         break;
-        case yaml_node_t::null:
+        case node_t::null:
             os << kw_null;
         break;
-        case yaml_node_t::number:
+        case node_t::number:
             os << static_cast<const yaml_value_number&>(node).value_number;
         break;
-        case yaml_node_t::string:
+        case node_t::string:
             json::dump_string(os, static_cast<const yaml_value_string&>(node).value_string);
         break;
-        case yaml_node_t::unset:
+        case node_t::unset:
         default:
             ;
     }
@@ -832,7 +821,7 @@ const char* warning_multiple_documents =
 
 }
 
-std::string yaml_document_tree::dump_yaml() const
+std::string document_tree::dump_yaml() const
 {
     std::ostringstream os;
 
@@ -847,7 +836,7 @@ std::string yaml_document_tree::dump_yaml() const
     return os.str();
 }
 
-std::string yaml_document_tree::dump_json() const
+std::string document_tree::dump_json() const
 {
     if (mp_impl->m_docs.empty())
         return std::string();
@@ -865,6 +854,6 @@ std::string yaml_document_tree::dump_json() const
     return os.str();
 }
 
-}
+}}
 
 /* vim:set shiftwidth=4 softtabstop=4 expandtab: */
