@@ -7,12 +7,12 @@
 
 #include "orcus/spreadsheet/document.hpp"
 
-#include "orcus/spreadsheet/global_settings.hpp"
 #include "orcus/spreadsheet/sheet.hpp"
 #include "orcus/spreadsheet/shared_strings.hpp"
 #include "orcus/spreadsheet/styles.hpp"
 #include "orcus/spreadsheet/auto_filter.hpp"
 #include "orcus/spreadsheet/pivot.hpp"
+#include "orcus/spreadsheet/config.hpp"
 
 #include "orcus/pstring.hpp"
 #include "orcus/types.hpp"
@@ -25,6 +25,7 @@
 #include <ixion/model_context.hpp>
 #include <ixion/formula_name_resolver.hpp>
 #include <ixion/interface/table_handler.hpp>
+#include <ixion/config.hpp>
 
 #include <iostream>
 #include <fstream>
@@ -204,17 +205,22 @@ class table_handler : public ixion::iface::table_handler
                 return ixion::abs_range_t(ixion::abs_range_t::invalid);
 
             col_t col1_index = find_column(tab, col1_name, 0);
+            if (col1_index < 0)
+                return ixion::abs_range_t(ixion::abs_range_t::invalid);
+
             if (column_last != ixion::empty_string_id)
             {
                 pstring col2_name = get_string(column_last);
-
-                // column range table reference.
-                col_t col2_index = find_column(tab, col2_name, col1_index);
-                ixion::abs_range_t range = tab.range;
-                range.first.column = col1_index;
-                range.last.column = col2_index;
-                adjust_row_range(range, tab, areas);
-                return range;
+                if (!col2_name.empty())
+                {
+                    // column range table reference.
+                    col_t col2_index = find_column(tab, col2_name, col1_index);
+                    ixion::abs_range_t range = tab.range;
+                    range.first.column = col1_index;
+                    range.last.column = col2_index;
+                    adjust_row_range(range, tab, areas);
+                    return range;
+                }
             }
 
             // single column table reference.
@@ -273,11 +279,12 @@ struct document_impl
 
     document& m_doc;
 
+    document_config m_doc_config;
     string_pool m_string_pool;
     ixion::model_context m_context;
     date_time_t m_origin_date;
     sheet_items_type m_sheets;
-    import_styles* mp_styles;
+    styles m_styles;
     import_shared_strings* mp_strings;
     ixion::dirty_formula_cells_t m_dirty_cells;
 
@@ -291,8 +298,8 @@ struct document_impl
 
     document_impl(document& doc) :
         m_doc(doc),
-        mp_styles(new import_styles(m_string_pool)),
-        mp_strings(new import_shared_strings(m_string_pool, m_context, *mp_styles)),
+        m_styles(),
+        mp_strings(new import_shared_strings(m_string_pool, m_context, m_styles)),
         m_pivots(doc),
         mp_name_resolver(ixion::formula_name_resolver::get(ixion::formula_name_resolver_t::excel_a1, &m_context)),
         m_grammar(formula_grammar_t::xlsx_2007),
@@ -304,7 +311,6 @@ struct document_impl
     ~document_impl()
     {
         delete mp_strings;
-        delete mp_styles;
     }
 };
 
@@ -322,14 +328,14 @@ const import_shared_strings* document::get_shared_strings() const
     return mp_impl->mp_strings;
 }
 
-import_styles* document::get_styles()
+styles& document::get_styles()
 {
-    return mp_impl->mp_styles;
+    return mp_impl->m_styles;
 }
 
-const import_styles* document::get_styles() const
+const styles& document::get_styles() const
 {
-    return mp_impl->mp_styles;
+    return mp_impl->m_styles;
 }
 
 pivot_collection& document::get_pivot_collection()
@@ -352,6 +358,19 @@ const ixion::model_context& document::get_model_context() const
     return mp_impl->m_context;
 }
 
+const document_config& document::get_config() const
+{
+    return mp_impl->m_doc_config;
+}
+
+void document::set_config(const document_config& cfg)
+{
+    mp_impl->m_doc_config = cfg;
+    ixion::config ixion_cfg = mp_impl->m_context.get_config();
+    ixion_cfg.output_precision = cfg.output_precision;
+    mp_impl->m_context.set_config(ixion_cfg);
+}
+
 string_pool& document::get_string_pool()
 {
     return mp_impl->m_string_pool;
@@ -401,6 +420,12 @@ sheet* document::append_sheet(const pstring& sheet_name, row_t row_size, col_t c
 }
 
 sheet* document::get_sheet(const pstring& sheet_name)
+{
+    const sheet* sh = const_cast<const document*>(this)->get_sheet(sheet_name);
+    return const_cast<sheet*>(sh);
+}
+
+const sheet* document::get_sheet(const pstring& sheet_name) const
 {
     auto it = std::find_if(
         mp_impl->m_sheets.begin(), mp_impl->m_sheets.end(), find_sheet_by_name(sheet_name));
@@ -413,10 +438,8 @@ sheet* document::get_sheet(const pstring& sheet_name)
 
 sheet* document::get_sheet(sheet_t sheet_pos)
 {
-    if (static_cast<size_t>(sheet_pos) >= mp_impl->m_sheets.size())
-        return nullptr;
-
-    return &mp_impl->m_sheets[sheet_pos]->data;
+    const sheet* sh = const_cast<const document*>(this)->get_sheet(sheet_pos);
+    return const_cast<sheet*>(sh);
 }
 
 const sheet* document::get_sheet(sheet_t sheet_pos) const
@@ -583,22 +606,27 @@ void document::set_formula_grammar(formula_grammar_t grammar)
     mp_impl->m_grammar = grammar;
 
     ixion::formula_name_resolver_t resolver_type = ixion::formula_name_resolver_t::unknown;
+    char arg_sep = 0;
 
     switch (mp_impl->m_grammar)
     {
         case formula_grammar_t::xls_xml:
             resolver_type = ixion::formula_name_resolver_t::excel_r1c1;
+            arg_sep = ',';
             break;
         case formula_grammar_t::xlsx_2007:
         case formula_grammar_t::xlsx_2010:
             resolver_type = ixion::formula_name_resolver_t::excel_a1;
+            arg_sep = ',';
             break;
         case formula_grammar_t::ods:
             resolver_type = ixion::formula_name_resolver_t::odff;
+            arg_sep = ';';
             break;
         case formula_grammar_t::gnumeric:
             // TODO : Use Excel A1 name resolver for now.
             resolver_type = ixion::formula_name_resolver_t::excel_a1;
+            arg_sep = ',';
             break;
         default:
             mp_impl->mp_name_resolver.reset();
@@ -610,6 +638,11 @@ void document::set_formula_grammar(formula_grammar_t grammar)
     {
         mp_impl->mp_name_resolver =
             ixion::formula_name_resolver::get(resolver_type, &mp_impl->m_context);
+
+        ixion::config cfg = mp_impl->m_context.get_config();
+        cfg.sep_function_arg = arg_sep;
+        cfg.output_precision = mp_impl->m_doc_config.output_precision;
+        mp_impl->m_context.set_config(cfg);
     }
 }
 
